from time import time
from Products.PortalTransforms.interfaces import itransform
from StringIO import StringIO
from zipfile import ZipFile
from OFS.Folder import Folder
from OFS.Image import File
from zLOG import LOG, INFO
import config

class BlueDCSDocTransform:
    """Transform which ...whatever..."""
    __implements__ = itransform
    __name__ = "bluedcs_office2html"
    inputs = (  'application/vnd.oasis.opendocument.spreadsheet',
                'application/vnd.oasis.opendocument.text',
                'application/vnd.oasis.opendocument.presentation',
                'application/msword',
                #'application/msexcel',
                #'application/vnd.ms-word',
                'application/vnd.ms-excel',
                'application/vnd.stardivision.writer',
                'application/vnd.sun.xml.writer',
                'application/vnd.stardivision.calc',
                'application/vnd.sun.xml.calc',

                'application/pdf',
                'application/vnd.ms-powerpoint',
                'application/vnd.stardivision.draw',
                'application/vnd.stardivision.impress',
                'application/vnd.sun.xml.draw',
                'application/vnd.sun.xml.impress',
                'application/vnd.sun.xml.writer.global',
                #'application/x-vnd.oasis.opendocument.spreadsheet',
                #'application/wordperfect',
                #'image/x-eps',
                #'image/vnd.dxf',

                )
    output = "text/html"

    def __init__(self, name='BlueDCS office_to_html'):
        if name:
            self.__name__ = name

    def name(self):
        return self.__name__

    def convert(self, data, idata, context, usedby=None, filename=None, **kwargs):
        """convert the data, store the result in idata and return that
        optional argument filename may give the original file name of received data
        additional arguments given to engine's convert, convertTo or __call__ are
        passed back to the transform

        The object on which the translation was invoked is available as context
        (default: None)
        """
        mainfile='index.html'
        
        t=time()
        unit = context.getField(usedby).getBaseUnit(context)
        datahash=hash(data)
        if getattr(unit,'raw_hash',0) == hash(data) and hasattr(unit,'rendered'):
            idata.setData(unit.rendered)
            return idata
        
        dcstool=context.portal_bluedcs
        LOG('BlueDCS: Transforms',INFO, 'started conversion at:  %s' % (time()-t))
        r=dcstool.convert(data,'HTML', filename, mainfile)
        LOG('BlueDCS: Transforms',INFO, 'finished conversion at: %s' % (time()-t))
        zip=ZipFile(StringIO(r))
        subObjects={}
        
        if zip.namelist():
            #stuff in the subobjects permanently
            #setattr(unit,annotation_folder_name,Folder())
            container=self.getLocalDCSStorage(context,usedby,reset=True) #getattr(unit,annotation_folder_name)
            for fe in zip.namelist():
                if fe != mainfile:
                    #subObjects[fe]=zip.read(fe)
                    file=File(fe,fe,zip.read(fe))
                    container._setObject(fe,file)
                
        index_html=zip.read(mainfile)
        index_html=index_html.replace('IMG SRC="','IMG SRC="resolveuid/%s/%s/%s/' % (context.UID(), config.annotation_folder_name, usedby))
        index_html=index_html.replace('embed src="','embed src="resolveuid/%s/%s/%s/' % (context.UID(), config.annotation_folder_name, usedby))
        idata.setData(index_html)
        unit.rendered=index_html
        unit.raw_hash=datahash
        idata.setSubObjects(subObjects)
        
        LOG('BlueDCS: Transforms',INFO, 'finished transform at: %s' % (time()-t))
        return idata

    def getLocalDCSStorage(self,context,usedby,reset=False):
        '''checks if the local storage for the binary data is here
        '''
        #import pdb;pdb.set_trace()
        if not hasattr(context.aq_base,config.annotation_folder_name):
            setattr(context,config.annotation_folder_name,Folder())
            
        folder=getattr(context,config.annotation_folder_name)
        
        if reset and hasattr(folder.aq_base,usedby):
            folder.manage_delObjects(usedby)
        
        if not hasattr(folder.aq_base,usedby):
            folder._setObject(usedby,Folder())
            
        return getattr(folder,usedby)
        
        
        
def register():
    return BlueDCSDocTransform()
